/*
 * main.c - main hdw-sniff
 *
 * Copyright (C) 2004 hackbard@hackdaworld.dyndns.org
 *
 */

#include "main.h"

void parse_package(unsigned char *ptr,const struct pcap_pkthdr *pcap_header,const unsigned char *package);

/* functions */

int get_user_event(t_info *info) {
  char event;
  if(read(0,&event,1)!=1) {
   perror("reading user interaction failed");
   return -23;
  }
  printf("user event: %c ",event);
  if(event=='h') {
   info->mode^=MODE_HEXOUT;
   printf("- hex output: %c\n",info->mode&MODE_HEXOUT?'a':'n');
  }
  if(event=='a') {
   info->mode^=MODE_ASCIIOUT;
   printf("- ascii output: %c\n",info->mode&MODE_ASCIIOUT?'a':'n');
  }
  if(event=='q') {
   info->mode|=MODE_QUIT;
   printf("- shutting down!\n");
  }

  return 23;
}

int usage(void) {
  puts("usage: hdw-sniff <options>");
  puts("\toptions:");
  puts("\t\t-m <mode> \tmonitor and/or wlanng");
  puts("\t\t-d <device> \twlan0,eth0");
  puts("\t\t-l <logfile>");
  puts("\t\t-k <key> \t(string)");
  puts("\t\t-h \tdisplay this help message");
  puts("");
}

int hop_channel(t_info *info) {
 
  struct iwreq iwreq;

  if(info->current_channel>=CHANNEL_MAX) info->current_channel=1;
  memset(&iwreq,0,sizeof(iwreq));
  strcpy(iwreq.ifr_name,info->device);
  iwreq.u.freq.e=0;
  iwreq.u.freq.m=info->current_channel;
  if(ioctl(info->channel_hop_fd,SIOCSIWFREQ,&iwreq)<0) {
    puts("unable to hop channel");
    perror("ioctl");
    return -23;
  }
 ++(info->current_channel);
 return 23;
}

int main(int argc, char **argv) {

  t_info info;
  int pcap_fd;
  fd_set fds;
  struct timeval hop_f;
  int i;
  char sys_call[MAX_SYSCALL_CHARS];
  char pcap_error[PCAP_ERRBUF_SIZE];

  memset(&info,0,sizeof(t_info));
 
 /* parse arguments */
 for(i=1;i<argc;i++)
 {
  if(argv[i][0]=='-')
  {
   switch(argv[i][1])
   {
    case 'h':
     usage();
    case 'm':
     if(!strncmp(argv[i+1],"monitor",7)) {
      info.mode|=MODE_MONITOR;
      puts("will go to monitor mode.");
     }
     else if(!strncmp(argv[i+1],"wlanng",6)) {
      info.mode|=MODE_WLANNG;
      puts("expecting wlanng header in package.");
     }
     else {
      printf("unknown mode: %s\n",argv[1]);
      return -23;
     }
     ++i;
     break;
    case 'l':
     if ((info.logfile_fd=open(argv[i+1],O_RDWR|O_CREAT))!=0)
      printf("logfile -> %s\n",argv[i+1]);
     else
      puts("warning: can't write to logfile.");
     ++i;
     break;
    case 'd':
     strncpy(info.device,argv[i+1],MAX_DEV_CHARS);
     ++i;
     break;
    default:
     usage();
     return -23;
   }
  } else {
   usage();
   return -23;
  }
 }

 /* setting up device */
 if(info.mode&MODE_MONITOR) {
  sprintf(sys_call,"iwconfig %s mode monitor",info.device);
  puts("set monitoring mode ...");
  system(sys_call);
 }
 sprintf(sys_call,"ifconfig %s up",info.device);
 puts("setting up device ...");
 system(sys_call);

 /* pcap */
 if((info.pcap_handle=pcap_open_live(info.device,BUFSIZ,1,-1,pcap_error))==NULL)
 {
  printf("%s: %s\n",argv[0],pcap_error);
  return -23;
 }
 pcap_fd=pcap_fileno(info.pcap_handle);
 /* -> non blocking? */

 /* socket fd for channel hopping */
 info.channel_hop_fd=socket(AF_INET,SOCK_DGRAM,0);

 /* parse packages until user breaks */
 while(!(info.mode&MODE_QUIT))
 {
  /* watch pcap_fd and stdin (reading) */
  FD_ZERO(&fds);
  FD_SET(pcap_fd,&fds);
  FD_SET(0,&fds);
  hop_f.tv_sec=HOP_SEC;
  hop_f.tv_usec=HOP_USEC;

  if(select(pcap_fd+1,&fds,NULL,NULL,&hop_f)) {
   if(FD_ISSET(0,&fds)) 
    get_user_event(&info);
   else if(FD_ISSET(pcap_fd,&fds))
    pcap_dispatch(info.pcap_handle,-1,parse_package,(unsigned char *)&info);
   else
    hop_channel(&info);
  }
 }

 puts("");
 puts("");
 puts("thanks for using hdw-sniff (C) 2005 hackbard");
 puts("");
 puts("bugreports: hackbard@hackdaworld.dyndns.org");

 return 23;

}
